/*
Copyright (c) 2008-2010 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this
 list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
 this list of conditions and the following disclaimer in the documentation
 and/or other materials provided with the distribution.
* Neither the name of Nokia Corporation nor the names of its contributors
 may be used to endorse or promote products derived from this software
 without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Description: Contains the definition of the CContainer class. 
*/


/**
 @file 
*/
#ifndef __CONSUMER_H__
#define __CONSUMER_H__

#include "globals.h"
#include "queue.h"

/**
This class owns the consumer thread.
It provides an interface to remove tokens from the CQueue::iArray queue.
@see CQueue.
@see CPeriodic.
*/
class CConsumer : public CBase
	{
public:
	static CConsumer* NewL(CConsoleBase* aConsole,CQueue* aTokens);
	void ConstructL(CConsoleBase* aConsole,CQueue* aTokens);
	static TInt ConsThreadFunc(TAny* aPtr);
	void ResumeCons();
	void Consume();
	~CConsumer();
	static TInt RemoveFunction(TAny* aPtr);
private:
	CConsumer();
private:
	/**
	The pointer to a CQueue object.
	Both CProducer and the CConsumer classes share the same object of the CQueue class.
	@see CProducer::iQueue.
	*/
	CQueue* iQueue;
	/**
	The consumer thread.
	It is invoked periodically by the CConsumer::iPeriodicCons object, which is an instance of the CPeriodic class.
	*/
	RThread iConsThread;
	/**
	The CPeriodic object.
	It runs the consumer thread (CConsumer::iConstThread) at regular intervals.
	@see CConsumer::ConstructL().
	*/
	CPeriodic* iPeriodicCons;
	/**
	The user console.
	*/
	CConsoleBase* iConsole;
	};

#endif
